﻿/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Filters
{
    public class FilterMagnifier : IFilter
    {
        frmFilterMagnifier form = new frmFilterMagnifier();
        Bitmap srcBitmap, destBitmap;

        ProgressEventHandler pEvent;
        FinishedEventHandler fEvent;


        public void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            srcBitmap = (Bitmap)bitmap.Clone();
            destBitmap = bitmap;
            pEvent = progressEvent;
            fEvent = finishedEvent;

            form.OwnFilter = this;
            form.PreviewImage = srcBitmap;

            DialogResult dR = form.ShowDialog();
            if (dR != DialogResult.OK) Restore();
        }

        public void Restore()
        {
            BitmapOperations.CopyBitmapToBitmap(srcBitmap, destBitmap);
            fEvent(this, new EventArgs());
        }


        public unsafe void RenderImage(int x, int y, int r, int scale, bool highQuality)
        {
            TimeMeasure tM = new TimeMeasure();
            pEvent(this, ProgressEventArgs.InProgress);

            BitmapData bDS = srcBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = destBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pSS = pS;
            byte* pD = (byte*)bDD.Scan0;
            int stride = bDD.Stride;
            int offset = stride - bDD.Width * 3;

            for (int i = 0; i < bDD.Height; ++i)
            {
                if (i % (bDD.Height / 10) == 0)
                {
                    pEvent(this, new ProgressEventArgs((float)i / bDD.Height));
                }

                for (int j = 0; j < bDD.Width; ++j)
                {
                    int d = (x - j) * (x - j) + (y - i) * (y - i);
                    double dD = Math.Sqrt(d);
                    
                    if (d < r * r)
                    {
                        if (highQuality) // magas minoseg (2×2 bilinear minta)
                        {
                            int sR = 0, sG = 0, sB = 0;

                            for (int sY = 1; sY <= 2; ++sY)
                            {
                                for (int sX = 1; sX <= 2; ++sX)
                                {
                                    float newX = (float)(x + (j - x) / (Math.Sqrt((double)(r - dD) / r) * scale + 1)) + (sY - 1) / 4.0f;
                                    float newY = (float)(y + (i - y) / (Math.Sqrt((double)(r - dD) / r) * scale + 1)) + (sY - 1) / 4.0f;

                                    if (newX >= 0 && newX < bDD.Width && newY >= 0 && newY < bDD.Height)
                                    {
                                        sB += getBilinearSample(bDS, newX, newY, 0);
                                        sG += getBilinearSample(bDS, newX, newY, 1);
                                        sR += getBilinearSample(bDS, newX, newY, 2);
                                    }
                                }
                            }

                            pD[0] = (byte)(sB / 4);
                            pD[1] = (byte)(sG / 4);
                            pD[2] = (byte)(sR / 4);                           
                        }
                        else // alacsonyabb minoseg
                        {
                            int newX = (int)(x + (j - x) / (Math.Sqrt((double)(r - dD) / r) * scale + 1));
                            int newY = (int)(y + (i - y) / (Math.Sqrt((double)(r - dD) / r) * scale + 1));
                            
                            if (newX >= 0 && newX < bDD.Width && newY >= 0 && newY < bDD.Height)
                            {
                                pD[0] = pSS[newY * stride + newX * 3];
                                pD[1] = pSS[newY * stride + newX * 3 + 1];
                                pD[2] = pSS[newY * stride + newX * 3 + 2];
                            }
                            else
                            {
                                pD[0] = pD[1] = pD[2] = 0;
                            }
                        }
                    }
                    else
                    {
                        pD[0] = pS[0];
                        pD[1] = pS[1];
                        pD[2] = pS[2];
                    }

                    pD += 3;
                    pS += 3;
                }
                pD += offset;
                pS += offset;
            }

            srcBitmap.UnlockBits(bDS);
            destBitmap.UnlockBits(bDD);

            pEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            fEvent(this, new EventArgs());
        }

        // http://en.wikipedia.org/wiki/Bilinear_interpolation
        private unsafe byte getBilinearSample(BitmapData bD, float x, float y, int channelOffset)
        {
            byte* p = (byte*)bD.Scan0;
            int iX = (int)Math.Floor((double)x);
            int iY = (int)Math.Floor((double)y);
            
            int f00 = p[iY * bD.Stride + iX * 3 + channelOffset];
            int f01 = p[(iY + 1) * bD.Stride + iX * 3 + channelOffset];
            int f10 = p[iY * bD.Stride + (iX + 1) * 3 + channelOffset];
            int f11 = p[(iY + 1) * bD.Stride + (iX + 1) * 3 + channelOffset];

            int s = (int)(f00 + (f10 - f00) * (x - iX) + (f01 - f00) * (y - iY) + (f00 - f10 - f01 + f11) * (x - iX) * (y - iY));
            return (byte)s;
        }

    }
}
