/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace Filters
{
    public partial class frmFilters : Form
    {
        float pictureScale = 1.0f;

        Image origPicture; // eredeti kep, a kep visszaallitasahoz
        Bitmap bM; // a szurot erre alkalmazzuk

        string initialDirectory;

        public frmFilters()
        {
            InitializeComponent();
        }


        private void enableControls()
        {
            miSave.Enabled = true;
            miPictureProperties.Enabled = true;
            miEdit.Enabled = true;
            miFilters.Enabled = true;
        }

        /// <summary>
        /// Szuro alkalmazasa a kepre
        /// </summary>
        /// <param name="filter">kivalaszott szuro</param>
        private void renderFilter(IFilter filter)
        {
            // csak ha van megmegnyitva kep
            if (bM != null) filter.Render(bM, filter_InProgress, filter_Finished);
        }


        private void frmFilters_Load(object sender, EventArgs e)
        {
            initialDirectory = Application.StartupPath;
        }


        /// <summary>
        /// Kepfajl betoltese
        /// </summary>
        /// <param name="fileName">Fajlnev eleresi uttal</param>
        private void loadPicture(string fileName)
        {
            try
            {
                origPicture = Image.FromFile(fileName);
                if (origPicture.PixelFormat != PixelFormat.Format24bppRgb && origPicture.PixelFormat!=PixelFormat.Format32bppRgb) throw new Exception("Nem 24 bites vagy 32 bites rgb kpformtum.");
                bM = (Bitmap)origPicture.Clone();
                pictureBox.Image = bM; // ujrarajzolaskor a Bitmap-et hasznalja
                pictureBox.Size = new Size((int)(bM.Width * pictureScale), (int)(bM.Height * pictureScale));
                lblRenderTime.Text = "";
                lblFileName.Text = fileName;
                enableControls();
            }
            catch (Exception e)
            {
                MessageBox.Show("Hibs kpfjl!\nMessage: " + e.Message + "\nSource: " + e.Source, "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                // ...
            }
        }

        /// <summary>
        /// Kep mentese
        /// </summary>
        /// <param name="fileName">Fajlnev eleresi uttal</param>
        /// <param name="filterIndex">Fajltipus</param>
        private void savePicture(string fileName, int filterIndex) 
        {
            ISaveImage fileSaver = null;
            switch (filterIndex)
            {
                case 1: // JPEG kepfajl
                    fileSaver = new SaveJPEG();
                    fileSaver.SaveImage(fileName, bM);
                    break;
                case 2: // BMP kepfajl
                    fileSaver = new SaveBMP();
                    fileSaver.SaveImage(fileName, bM);
                    break;
                default:
                    break;
            }
        }


        private void filter_InProgress(object sender, ProgressEventArgs e)
        {
            progressBar.Value = (int)(e.Progress * 100);
            lblRenderTime.Text = e.Message;
            statusStrip.Refresh();
        }

        private void filter_Finished(object sender, EventArgs e)
        {
            pictureBox.Refresh();
        }


        #region MENUPONTOK

        /// <summary>
        /// Kepfajl megnyitasa
        /// </summary>
        private void miOpen_Click(object sender, EventArgs e)
        {
            openFileDialog.InitialDirectory = initialDirectory;
            DialogResult dR = openFileDialog.ShowDialog();

            if (dR == DialogResult.OK)
            {
                loadPicture(openFileDialog.FileName); // kep betoltese
                initialDirectory = new System.IO.FileInfo(openFileDialog.FileName).DirectoryName; // legkozelebb is ebbol a konyvtarbol valaszthassunk
            }
        }


        private void miSave_Click(object sender, EventArgs e)
        {
            // ...
            DialogResult dR = saveFileDialog.ShowDialog();

            if (dR == DialogResult.OK)
            {
                savePicture(saveFileDialog.FileName, saveFileDialog.FilterIndex);
            }
        }


        /// <summary>
        /// Kep visszaallitasa
        /// </summary>
        private void miPictureUndo_Click(object sender, EventArgs e)
        {
            if (bM == null) return; // nem volt megnyitva kep
            // kep megnyitasakori allapot visszaallitasa
            pictureBox.Image = null;
            bM.Dispose();
            bM = (Bitmap)origPicture.Clone();
            pictureBox.Image = bM;
            lblRenderTime.Text = "";
        }


        private void miInverz_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterInvert());
        }


        private void miConvolutionFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterConvolution());
        }


        private void miGreyscaleFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterGreyscale());
        }


        private void miHorizontalMirrorFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterVerticalMirror());
        }


        private void miVerticalMirrorFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterHorizontalMirror());
        }


        private void miRotate180Filter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterRotate180());
        }


        private void miTwirlFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterTwirl());
        }


        private void miDiffusionFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterDiffusion());
        }


        private void miWaveFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterWaves());
        }


        private void miExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }


        private void miAverageFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterAverage());
        }


        private void miMagnifierFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterMagnifier());
        }


        private void miMosaicFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterMosaic());
        }


        private void miTresholdFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterTreshold());
        }


        private void miGreyscaleSafeCodeFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterGreyscaleSafecode());
        }


        private void miAddNoiseFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterAddNoise());
        }


        private void miColorBalanceFilter_Click(object sender, EventArgs e)
        {
            renderFilter(new FilterColorBalance());
        }


        private void miBlurFilter_Click(object sender, EventArgs e)
        {

        }


        private void miGaussBlurFilter_Click(object sender, EventArgs e)
        {

        }

        private void miBoxBlurFilter_Click(object sender, EventArgs e)
        {

        }

        private void miLastFilter_Click(object sender, EventArgs e)
        {

        }

        private void miFiltersWindow_Click(object sender, EventArgs e)
        {

        }

        private void miRemoveNoiseFilter_Click(object sender, EventArgs e)
        {

        }


        private void miPictureFromClipboard_Click(object sender, EventArgs e)
        {
            if (Clipboard.ContainsImage())
            {
                try
                {
                    origPicture = Clipboard.GetImage();
                    if (origPicture.PixelFormat != PixelFormat.Format24bppRgb && origPicture.PixelFormat != PixelFormat.Format32bppRgb) throw new Exception("Nem 24 bites vagy 32 bites rgb kpformtum.");
                    bM = (Bitmap)origPicture.Clone();
                    pictureBox.Image = bM; // ujrarajzolaskor a Bitmap-et hasznalja
                    pictureBox.Size = new Size((int)(bM.Width * pictureScale), (int)(bM.Height * pictureScale));
                    lblRenderTime.Text = "";
                    lblFileName.Text = "Vglaprl beillesztve";
                    enableControls();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Hibs kpfjl!\nMessage: " + ex.Message + "\nSource: " + ex.Source, "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    // ...
                }
            }
            else
            {
                MessageBox.Show("Nincs kp a vglapon!");
            }
        }


        private void miPictureProperties_Click(object sender, EventArgs e)
        {
            frmPictureProperties form = new frmPictureProperties(lblFileName.Text, bM);
            form.ShowDialog();
        }


        private void miXPercent_Click(object sender, EventArgs e)
        {
            // ... <- megoldani, hogy nagyitaskor ne legyen bilinear
            // ... <- hiba: kifejezetten nagy kepek nagyitasanal elszall

            scaleStrip.Text = ((ToolStripMenuItem)sender).Text;
            int scale = Convert.ToInt32(((ToolStripMenuItem)sender).Text.Split(' ')[0]);
            pictureScale = (float)scale / 100;

            if (bM != null)
            {
                pictureBox.Size = new Size((int)(bM.Width * pictureScale), (int)(bM.Height * pictureScale));
            }
        }


        private void miSettings_Click(object sender, EventArgs e)
        {
            // ...
            frmSettings form = new frmSettings();
            form.ShowDialog();
        }


        #endregion


    }
}